﻿/*Copyright (c) 2018 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using UnityEditor;
using UnityEditor.SceneManagement;
using Sigian.Experimental;

public class UtilityWindow : EditorWindow {

    private float u_timescale = 1f;
    private float horizontalMargin = 15f;

    public static float autoSaveDelay = 5f;
    static double nextSave = 0f;

    private bool isAutoSaveEnabled = false;
    public static bool isSmartNamingEnabled = true;
    public static bool isParentCenteringEnabled = true;

    //SESSION
    private bool s_isRunning = false;
    private float s_initialTime = 1;
    private float s_progressTime;
    private bool s_shouldCongrat = true;

    //Toolbar
    public int toolbarInt = 0;
    public string[] toolbarTitles = new string[] { "Features", "Options", "Index" };

    //Hotkey hints
    public TextAsset allHotkeys;
    private string[] choppedHotkeys;
    Vector2 scrollPosHK;

    private void Awake()
    {
        InitDataBases();
    }

    [MenuItem("Sigian/Utility Window")]
    static void InitWindow()
    {
        UtilityWindow window = (UtilityWindow)GetWindow(typeof(UtilityWindow));
        window.Show();
    }

    public void InitDataBases()
    {
        allHotkeys = AssetDatabase.LoadAssetAtPath<TextAsset>("Assets/EditorResources/hotkeys.txt");
        choppedHotkeys = allHotkeys.text.Split('#');
    }

    private void Update()
    {
        Time.timeScale = u_timescale;

        if (s_isRunning)
        {
            s_progressTime += Time.deltaTime;

            if(s_progressTime >= s_initialTime)
            {
                s_isRunning = false;
                s_initialTime /= 3600;

                if (s_shouldCongrat)
                {
                    Congrats.InitWizard();
                }

                s_shouldCongrat = true;
            }
        }
    }
   
    private void OnGUI()
    {
        GUILayout.Label("Frames Per Second:" + (1f/Time.unscaledDeltaTime).ToString("00"), EditorStyles.largeLabel);


        if (!EditorApplication.isPlaying || EditorApplication.isPaused)
        {
            if (GUILayout.Button("\nPLAY SCENE\n"))
            {
                if (EditorApplication.isPaused)
                {
                    EditorApplication.isPaused = false;
                }
                else
                {
                    EditorApplication.isPlaying = true;
                }             
            }
        }
        else
        {
            EditorGUILayout.BeginHorizontal();

            if(GUILayout.Button("\nSTOP SCENE\n"))
            {
                EditorApplication.isPlaying = false;
            }

            if(GUILayout.Button("\nPAUSE SCENE\n"))
            {
                EditorApplication.isPaused = true;
            }

            EditorGUILayout.EndHorizontal();
        }
        

        GUILayout.Space(horizontalMargin*1.5f);

        //Toolbars
        toolbarInt = GUILayout.Toolbar(toolbarInt, toolbarTitles);

        //Tool1 (features)
        using(var group1 = new EditorGUILayout.FadeGroupScope(Convert.ToSingle(toolbarInt==0)))
        {
            if (group1.visible)
            {
                /*EditorGUILayout.Space();

                if (GUILayout.Button("SETUP"))
                {
                    InitDataBases();
                }*/

                EditorGUILayout.Space();

                GUILayout.BeginHorizontal();

                if (GUILayout.Button("SAVE YOUR \nWORK"))
                {
                    EditorSceneManager.SaveScene(EditorSceneManager.GetActiveScene());
                    nextSave = EditorApplication.timeSinceStartup + (autoSaveDelay * 60);
                }

                double timesincestart = EditorApplication.timeSinceStartup;
                int[] timearranged = timesincestart.ToHoursMinutesSeconds();
                     
                if (timearranged[0] > 0)
                {
                    GUILayout.Box("You have been working for " + timearranged[0] + " h and " + timearranged[1] + " min.", EditorStyles.boldLabel);
                }
                else
                {
                    GUILayout.Box("You have been working for " + timearranged[1] + " minutes.", EditorStyles.boldLabel);
                }

                GUILayout.EndHorizontal();

                EditorGUILayout.Space();

                GUILayout.BeginHorizontal();
                u_timescale = EditorGUILayout.Slider("Timescale", u_timescale, 0.1f, 2f);
                GUILayout.EndHorizontal();

                EditorGUILayout.Space();

                EditorGUILayout.BeginHorizontal();

                if (GUILayout.Button("Open 'Select all of tags' wizard", GUILayout.Height(30)))
                {
                    SelectAllOfTags.LaunchWizard();
                }

                if (GUILayout.Button("Regroup selected in a parent", GUILayout.Height(30)))
                {
                    RegroupItems.Regroup();
                }

                EditorGUILayout.EndHorizontal();

                EditorGUILayout.BeginHorizontal();

                if (GUILayout.Button("Change Tag and/or Layer", GUILayout.Height(30)))
                {
                    ChangeTagAndLayer.LaunchWizard();
                }

                if(GUILayout.Button("Calculate Offset", GUILayout.Height(30)))
                {
                    OffsetCalculator.LaunchWizard();
                }

                EditorGUILayout.EndHorizontal();

                EditorGUILayout.BeginHorizontal();

                if (GUILayout.Button("Create spline", GUILayout.Height(30)))
                {
                    CreateSpline();
                }

                if (GUILayout.Button("Build Spline", GUILayout.Height(30)))
                {
                    SplineBuilder.LaunchWizard();
                }

                EditorGUILayout.EndHorizontal();

                GUILayout.FlexibleSpace();


                //Session Timer button and gauge
                if (!s_isRunning)
                {
                    if(GUILayout.Button("Begin Session Timer", GUILayout.Height(38)))
                    {
                        BeginSession(s_initialTime);
                    }
                }
                else
                {
                    Rect barRect = new Rect(3, 290, 350, 35);

                    EditorGUI.ProgressBar(barRect, (s_progressTime / s_initialTime), "\nTime progress. Don't give up!");

                    GUILayout.BeginArea(new Rect(365, 300, 35, 35));
                    if(GUILayout.Button("X"))
                    {
                        s_progressTime = s_initialTime;
                        s_shouldCongrat = false;
                    }
                    GUILayout.EndArea();
                }
                
            }
            
        }    

        //Tool2 (options)
        using(var group2 = new EditorGUILayout.FadeGroupScope(Convert.ToSingle(toolbarInt == 1)))
        {
            if (group2.visible)
            {
                EditorGUILayout.Space();
                EditorGUILayout.LabelField("Autosaving feature", EditorStyles.whiteMiniLabel);
                EditorGUILayout.BeginHorizontal();
                EditorGUILayout.PrefixLabel("Enable Autosave");
                isAutoSaveEnabled = EditorGUILayout.Toggle(isAutoSaveEnabled);
                EditorGUILayout.EndHorizontal();

                if (isAutoSaveEnabled)
                {
                    EditorGUILayout.BeginHorizontal();
                    EditorGUILayout.PrefixLabel("Frequency (minutes)");
                    autoSaveDelay = EditorGUILayout.FloatField(autoSaveDelay);
                    EditorGUILayout.EndHorizontal();
                }

                EditorGUILayout.Space();
                EditorGUILayout.LabelField("Enable this to allow parent to auto-rename itself in certain cases", EditorStyles.whiteMiniLabel);
                EditorGUILayout.BeginHorizontal();
                EditorGUILayout.PrefixLabel("Smart Parent Renaming");
                isSmartNamingEnabled = EditorGUILayout.Toggle(isSmartNamingEnabled);
                EditorGUILayout.EndHorizontal();

                EditorGUILayout.Space();
                EditorGUILayout.LabelField("Enable this to allow parent to replace itself to the middle of the group", EditorStyles.whiteMiniLabel);
                EditorGUILayout.BeginHorizontal();
                EditorGUILayout.PrefixLabel("Parent placing");
                isParentCenteringEnabled = EditorGUILayout.Toggle(isParentCenteringEnabled);
                EditorGUILayout.EndHorizontal();

                EditorGUILayout.Space();
                EditorGUILayout.BeginHorizontal();
                EditorGUILayout.PrefixLabel("Session time (hour)");
                s_initialTime = EditorGUILayout.FloatField(s_initialTime);
                EditorGUILayout.EndHorizontal();

            }
        }

        //Tool3 (index)
        using(var group3 = new EditorGUILayout.FadeGroupScope(Convert.ToSingle(toolbarInt == 2)))
        {
            if (group3.visible)
            {
                //Add content for group 3
                EditorGUILayout.Space();
                EditorGUILayout.LabelField("Hotkeys", EditorStyles.boldLabel);

                scrollPosHK = EditorGUILayout.BeginScrollView(scrollPosHK, EditorStyles.label);

                for(int i = 0; i < choppedHotkeys.Length; i++)
                {
                    //Display the hotkeys from the string array
                    EditorGUILayout.LabelField(choppedHotkeys[i], EditorStyles.miniLabel);
                }

                EditorGUILayout.EndScrollView();
            }
        }

        //EndToolbars

        GUILayout.FlexibleSpace();
        GUILayout.Label("Remember: you're awesome", EditorStyles.centeredGreyMiniLabel);

        this.Repaint();

        if(EditorApplication.timeSinceStartup > nextSave && isAutoSaveEnabled)
        {
            nextSave = EditorApplication.timeSinceStartup + (autoSaveDelay * 60);
            Debug.Log("Autosaving...");
            EditorSceneManager.SaveScene(EditorSceneManager.GetActiveScene());
        }
    }

    public void BeginSession(float time)
    {
        s_isRunning = true;
        s_progressTime = 0f;
        s_initialTime = time * 3600;
    }
		
    public void CreateSpline()
    {
        var GO = new GameObject().AddComponent<Spline>();
        GO.name = "NEW SPLINE";
    }
}
