﻿/*Copyright (c) 2017 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using System;

namespace Sigian
{
    public static class ExtensionMethods
    {
        /// <summary>
        /// Returns a random element from this array.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="array"></param>
        /// <returns></returns>
        public static T GetRandom<T>(this T[] array)
        {
            return array[UnityEngine.Random.Range(0, array.Length)];
        }

        /// <summary>
        /// Returns a random element from this List.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="list"></param>
        /// <returns></returns>
        public static T GetRandom<T>(this List<T> list)
        {
            return list[UnityEngine.Random.Range(0, list.Count)];
        }

        /// <summary>
        /// Returns true if the array contains the specified element. Works for any type.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="array"></param>
        /// <param name="elem">The element you want to find in the array</param>
        /// <returns></returns>
        public static bool Contains<T>(this T[] array, T elem)
        {
            if (array.Length == 0)
                return false;

            else
            {
                for (int i = 0; i < array.Length; i++)
                {
                    if (array[i].Equals(elem))
                        return true;
                }
                return false;
            }
        }

        /// <summary>
        /// Returns true if all the elements of this array are the same.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="array"></param>
        /// <returns></returns>
        public static bool IsConsistent<T>(this T[] array)
        {
            int i = 1;

            for (i = 1; i < array.Length; i++)
            {
                if (!array[i].Equals(array[i - 1])) break;
            }

            return i == array.Length;
        }

        /// <summary>
        /// Returns true if all the elements of this List are the same.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="list"></param>
        /// <returns></returns>
        public static bool IsConsistent<T>(this List<T> list)
        {
            int i = 1;

            for (i = 1; i < list.Count; i++)
            {
                if (!list[i].Equals(list[i - 1])) break;
            }

            return i == list.Count;
        }

        /// <summary>
        /// Converts this array into a List.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="array"></param>
        /// <returns></returns>
        public static List<T> ToList<T>(this T[] array)
        {
            List<T> temp = new List<T>();

            foreach(T elem in array)
            {
                temp.Add(elem);
            }

            return temp;
        }

        public static void Fill<T>(this List<T> list, int amount, T value)
        {
            if (list == null)
                return;

            for(int i = 0; i < amount; i ++)
            {
                list.Add(value);
            }
        }

        /// <summary>
        /// Returns true if this string is Null or Empty.
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static bool IsNullOrEmpty(this string str)
        {
            return str == string.Empty || str == null;
        }

        /// <summary>
        /// Returns true if your array is null or has no objects in it.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="array"></param>
        /// <returns></returns>
        public static bool IsNullOrEmpty<T>(this T[] array)
        {
            return array == null || array.Length == 0;
        }

        /// <summary>
        /// Returns true if the number is even, false if odd.
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public static bool IsEven(this int value)
        {
            return (value % 2) == 0;
        }

		/// <summary>
		/// Determines if this int is multiple of the specified multiple.
		/// </summary>
		/// <returns><c>true</c> if is multiple of the specified predicate multiple; otherwise, <c>false</c>.</returns>
		/// <param name="predicate">Predicate.</param>
		/// <param name="multiple">Multiple.</param>
		public static bool IsMultipleOf(this int predicate, int multiple)
		{
			return (predicate % multiple) == 0;
		}

        /// <summary>
        /// Returns the word count of this string.
        /// </summary>
        /// <param name="sentence"></param>
        /// <returns></returns>
        public static int WordCount(this string sentence)
        {
            return sentence.Split(new char[] { ' ', '.', '?', '!' }, System.StringSplitOptions.RemoveEmptyEntries).Length;
        }

        /// <summary>
        /// Returns true if this string is composed exclusively of numbers.
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static bool IsNumeric(this string str)
		{
			float fl;
			return float.TryParse (str, out fl);
		}

        /// <summary>
        /// Use this to quickly calculate random outcomes. Set the propability between 0 and 1.
        /// </summary>
        /// <param name="boo"></param>
        /// <returns>Example: 0.6 will set a 60% chance to return true.</returns>
        public static void DiceRoll(this bool boo, float probability)
        {
            if (probability > 1 || probability < 0)
                throw new Exception("Probability should be set between 0 and 1!");

            boo = UnityEngine.Random.Range(0f, 1f) <= probability;
        }

        /// <summary>
        /// Sets the color of all pixels of this texture.
        /// </summary>
        /// <param name="texture"></param>
        /// <param name="color"></param>
        public static void SetColor(this Texture2D texture, Color color)
        {
            for (int y = 0; y < texture.height; y++)
            {
                for (int x = 0; x < texture.width; x++)
                {
                    texture.SetPixel(x, y, color);
                }
            }

            texture.Apply();
        }

        /// <summary>
        /// Use this to quickly check is this GameObject has a component type attached.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="GO"></param>
        /// <param name="component">The component type you want to check.</param>
        /// <returns></returns>
        public static bool HasComponent(this GameObject GO, Type component)
        {
            Component[] comps = GO.GetComponents(component);
            return comps.Length >= 0;
        }

        /// <summary>
        /// Shuffles an array to mix its elements.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="array"></param>
        /// <returns></returns>
        public static void Shuffle<T>(this T[] array)
        {
            int t = 0;
            int L = array.Length;
            T[] temp = new T[L];
            List<int> indexer = new List<int>();

            for(int i = 0; i < L; i++)
            {
                do
                {
                    t = UnityEngine.Random.Range(0, L);
                } while (indexer.Contains(t));

                temp[i] = array[t];
                indexer.Add(t);
            }
            for(int j = 0; j < L; j++)
            {
                array[j] = temp[j];
            }
        }

        /// <summary>
        /// Shuffles a List to mix its elements.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="list"></param>
        /// <returns></returns>
        public static void Shuffle<T>(this List<T> list)
        {
            int t = 0;
            int C = list.Count;
            List<T> temp = new List<T>();
            List<int> indexer = new List<int>();

            for(int i = 0; i < C; i++)
            {
                do
                {
                    t = UnityEngine.Random.Range(0, C);
                } while (indexer.Contains(t));

                indexer.Add(t);
                temp.Add(list[t]);
            }

            for(int j = 0; j < C; j++)
            {
                list[j] = temp[j];
            }
        }

        /// <summary>
        /// Merges an array into this one. Order isn't changed, second array objects come after.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="arr"></param>
        /// <param name="array">The array you want to put inside the other.</param>
        /// <returns></returns>
        public static void Push<T>(this T[] arr, T[] array)
        {
            T[] temp = new T[arr.Length + array.Length];

            for (int i = 0; i < (arr.Length + array.Length); i++)
            {
                if (i <= arr.Length) temp[i] = arr[i];
                else temp[i] = array[i - arr.Length];
            }

            arr = temp;
        }

        /// <summary>
        /// Merges a List into this one. Useful to avoid ugly for loops in your code.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="li"></param>
        /// <param name="list">The List you want to merge. Objects in it will come after the ones of the first.</param>
        /// <returns></returns>
        public static void Push<T>(this List<T> li, List<T> list)
        {
            List<T> temp = new List<T>();

            foreach (T obj in li)
                temp.Add(obj);

            foreach (T obj in list)
                temp.Add(obj);

            li = temp;
        }

        /// <summary>
        /// Useful if you want to append a string while keeping your code clear. Adds a space.
        /// </summary>
        /// <param name="str"></param>
        /// <param name="addlMessage">The additionnal message.</param>
        /// <returns></returns>
        public static void Append(this string str, string addlMessage)
        {
            str +=  (" " + addlMessage);
        }

        /// <summary>
        /// Generates a random seed composed of numbers. You need to set the length of the seed.
        /// </summary>
        /// <param name="seedLength">The length of the seed</param>
        /// <returns></returns>
        public static int GenerateRandomSeed(this int intt, int seedLength)
        {
            List<int> brokenSeed = new List<int>();

            for (int i = 0; i < seedLength; i++)
                brokenSeed.Add(UnityEngine.Random.Range(0, 9));

            int finalSeed = 0;

            for (int j = 0; j < brokenSeed.Count; j++)
                finalSeed += (brokenSeed[j] * (Mathf.FloorToInt(Mathf.Pow(10, j))));

            return finalSeed;
        }

        /// <summary>
        /// Returns true if this string contains a specific char.
        /// </summary>
        /// <param name="str"></param>
        /// <param name="character">The char you want to search for.</param>
        /// <returns></returns>
        public static bool ContainsChar(this string str, char character)
        {
            return str.ToCharArray().Contains(character);
        }

        /// <summary>
        /// Basically a very very long Vector3.forward
        /// </summary>
        /// <param name="Trans"></param>
        /// <returns></returns>
        public static Vector3 longForward(this Transform Trans)
        {
            return Trans.forward * 1000000000000;
        }

        /// <summary>
        /// Basically a very very long Vector3.up
        /// </summary>
        /// <param name="Trans"></param>
        /// <returns></returns>
        public static Vector3 longUp(this Transform Trans)
        {
            return Trans.up * 1000000000000;
        }

        /// <summary>
        /// Basically a very very long Vector3.right
        /// </summary>
        /// <param name="Trans"></param>
        /// <returns></returns>
        public static Vector3 longRight(this Transform Trans)
        {
            return Trans.right * 1000000000000;
        }

        /// <summary>
        /// Returns a new vector3 composed of all components rounded down.
        /// </summary>
        public static Vector3 Floor(this Vector3 vector)
        {
            return new Vector3(Mathf.Floor(vector.x), Mathf.Floor(vector.y), Mathf.Floor(vector.z));
        }

        /// <summary>
        /// Returns a new vector2 composed of the x and y coordinates.
        /// </summary>
        public static Vector2 XY(this Vector3 vector)
        {
            return new Vector2(vector.x, vector.y);
        }

        /// <summary>
        /// Returns a new vector2 composed of the x and z coordinates.
        /// </summary>
        public static Vector2 XZ(this Vector3 vector)
        {
            return new Vector2(vector.x, vector.z);
        }

        /// <summary>
        /// Returns a new vector2 composed of the y and z coordinates.
        /// </summary>
        public static Vector2 YZ(this Vector3 vector)
        {
            return new Vector2(vector.y, vector.z);
        }

        /// <summary>
        /// Sets the x position of a transform.
        /// </summary>
        /// <param name="value">The new x position.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetPosX(this Transform transform, float value)
        {
            transform.position = new Vector3(value, transform.position.y, transform.position.z);
            return transform;
        }

        /// <summary>
        /// Sets the y position of a transform.
        /// </summary>
        /// <param name="value">The new y position.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetPosY(this Transform transform, float value)
        {
            transform.position = new Vector3(transform.position.x, value, transform.position.z);
            return transform;
        }

        /// <summary>
        /// Sets the z position of a transform.
        /// </summary>
        /// <param name="value">The new z position.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetPosZ(this Transform transform, float value)
        {
            transform.position = new Vector3(transform.position.x, transform.position.y, value);
            return transform;
        }

        /// <summary>
        /// Sets the x position of a transform relative to the actual x position.
        /// </summary>
        /// <param name="value">The x position shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativePosX(this Transform transform, float value)
        {
            transform.position += new Vector3(value, 0, 0);
            return transform;
        }

        /// <summary>
        /// Sets the y position of a transform relative to the actual y position.
        /// </summary>
        /// <param name="value">The y position shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativePosY(this Transform transform, float value)
        {
            transform.position += new Vector3(0, value, 0);
            return transform;
        }

        /// <summary>
        /// Sets the z position of a transform relative to the actual z position.
        /// </summary>
        /// <param name="value">The z position shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativePosZ(this Transform transform, float value)
        {
            transform.position += new Vector3(0, 0, value);
            return transform;
        }

        /// <summary>
        /// Sets the x local position of a transform.
        /// </summary>
        /// <param name="value">The new x local position.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetLocalPosX(this Transform transform, float value)
        {
            transform.localPosition = new Vector3(value, transform.localPosition.y, transform.localPosition.z);
            return transform;
        }

        /// <summary>
        /// Sets the y local position of a transform.
        /// </summary>
        /// <param name="value">The new y local position.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetLocalPosY(this Transform transform, float value)
        {
            transform.localPosition = new Vector3(transform.localPosition.x, value, transform.localPosition.z);
            return transform;
        }

        /// <summary>
        /// Sets the z local position of a transform.
        /// </summary>
        /// <param name="value">The new z local position.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetLocalPosZ(this Transform transform, float value)
        {
            transform.localPosition = new Vector3(transform.localPosition.x, transform.localPosition.y, value);
            return transform;
        }

        /// <summary>
        /// Sets the x local position of a transform relative to the actual x local position.
        /// </summary>
        /// <param name="value">The x local position shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeLocalPosX(this Transform transform, float value)
        {
            transform.localPosition += new Vector3(value, 0, 0);
            return transform;
        }

        /// <summary>
        /// Sets the y local position of a transform relative to the actual y local position.
        /// </summary>
        /// <param name="value">The y local position shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeLocalPosY(this Transform transform, float value)
        {
            transform.localPosition += new Vector3(0, value, 0);
            return transform;
        }

        /// <summary>
        /// Sets the z local position of a transform relative to the actual z local position.
        /// </summary>
        /// <param name="value">The z local position shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeLocalPosZ(this Transform transform, float value)
        {
            transform.localPosition += new Vector3(0, 0, value);
            return transform;
        }

        /// <summary>
        /// Sets the x rotation of a transform.
        /// </summary>
        /// <param name="value">The new x rotation.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRotationX(this Transform transform, float value)
        {
            Vector3 euler = transform.rotation.eulerAngles;
            transform.rotation = Quaternion.Euler(value, euler.y, euler.z);
            return transform;
        }

        /// <summary>
        /// Sets the y rotation of a transform.
        /// </summary>
        /// <param name="value">The new y rotation.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRotationY(this Transform transform, float value)
        {
            Vector3 euler = transform.rotation.eulerAngles;
            transform.rotation = Quaternion.Euler(euler.x, value, euler.z);
            return transform;
        }

        /// <summary>
        /// Sets the z rotation of a transform.
        /// </summary>
        /// <param name="value">The new z rotation.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRotationZ(this Transform transform, float value)
        {
            Vector3 euler = transform.rotation.eulerAngles;
            transform.rotation = Quaternion.Euler(euler.x, euler.y, value);
            return transform;
        }

        /// <summary>
        /// Sets the x rotation of a transform relative to the actual x rotation.
        /// </summary>
        /// <param name="value">The x rotation shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeRotationX(this Transform transform, float value)
        {
            Vector3 euler = transform.rotation.eulerAngles;
            transform.rotation = Quaternion.Euler(euler.x + value, euler.y, euler.z);
            return transform;
        }

        /// <summary>
        /// Sets the y rotation of a transform relative to the actual y rotation.
        /// </summary>
        /// <param name="value">The y rotation shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeRotationY(this Transform transform, float value)
        {
            Vector3 euler = transform.rotation.eulerAngles;
            transform.rotation = Quaternion.Euler(euler.x, euler.y + value, euler.z);
            return transform;
        }

        /// <summary>
        /// Sets the z rotation of a transform relative to the actual z rotation.
        /// </summary>
        /// <param name="value">The z rotation shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeRotationZ(this Transform transform, float value)
        {
            Vector3 euler = transform.rotation.eulerAngles;
            transform.rotation = Quaternion.Euler(euler.x, euler.y, euler.z + value);
            return transform;
        }

        /// <summary>
        /// Sets the x local rotation of a transform.
        /// </summary>
        /// <param name="value">The new x local rotation.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetLocalRotationX(this Transform transform, float value)
        {
            Vector3 euler = transform.localRotation.eulerAngles;
            transform.localRotation = Quaternion.Euler(value, euler.y, euler.z);
            return transform;
        }

        /// <summary>
        /// Sets the y local rotation of a transform.
        /// </summary>
        /// <param name="value">The new y local rotation.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetLocalRotationY(this Transform transform, float value)
        {
            Vector3 euler = transform.localRotation.eulerAngles;
            transform.localRotation = Quaternion.Euler(euler.x, value, euler.z);
            return transform;
        }

        /// <summary>
        /// Sets the z local rotation of a transform.
        /// </summary>
        /// <param name="value">The new z local rotation.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetLocalRotationZ(this Transform transform, float value)
        {
            Vector3 euler = transform.localRotation.eulerAngles;
            transform.localRotation = Quaternion.Euler(euler.x, euler.y, value);
            return transform;
        }

        /// <summary>
        /// Sets the x local rotation of a transform relative to the actual x local rotation.
        /// </summary>
        /// <param name="value">The x local rotation shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeLocalRotationX(this Transform transform, float value)
        {
            Vector3 euler = transform.localRotation.eulerAngles;
            transform.localRotation = Quaternion.Euler(euler.x + value, euler.y, euler.z);
            return transform;
        }

        /// <summary>
        /// Sets the y local rotation of a transform relative to the actual y local rotation.
        /// </summary>
        /// <param name="value">The y local rotation shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeLocalRotationY(this Transform transform, float value)
        {
            Vector3 euler = transform.localRotation.eulerAngles;
            transform.localRotation = Quaternion.Euler(euler.x, euler.y + value, euler.z);
            return transform;
        }

        /// <summary>
        /// Sets the z local rotation of a transform relative to the actual z local rotation.
        /// </summary>
        /// <param name="value">The z local rotation shift value.</param>
        /// <returns>Returns the transform so you can chain the functions.</returns>
        public static Transform SetRelativeLocalRotationZ(this Transform transform, float value)
        {
            Vector3 euler = transform.localRotation.eulerAngles;
            transform.localRotation = Quaternion.Euler(euler.x, euler.y, euler.z + value);
            return transform;
        }


        #region SigianTagExtensions | SigianID Extensions

        /// <summary>
        /// The SigianTag component of this GameObject. If unequipped, a new one will be created.
        /// </summary>
        /// <param name="GO"></param>
        /// <returns></returns>
        public static SigianTagComponent sigianTag(this GameObject GO)
        {
            if (!GO.HasComponent(typeof(SigianTagComponent)))
            {
                GO.AddComponent<SigianTagComponent>();
            }

            return GO.GetComponent<SigianTagComponent>();
        }

        #endregion


    }

    namespace Experimental
    {
        public static class ExperimentalExtensions
        {
            /// <summary>
            /// Pretty broken and untested. Please refrain from using.
            /// </summary>
            /// <param name="number"></param>
            /// <returns></returns>
            public static string NumberToLetters(this int number)
            {
                if (number == 0)
                    return "zero";

                if (number == 1000)
                    return "one thousand";

                int units = number % 10;
                int tens = (number % 100 - units) / 10;
                int hundreds = (number % 1000 - number % 100) / 100;

                string[] unitsToLetters = { "", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten", "eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen", "eighteen", "nineteen" };
                string[] tensToLetters = { "", "ten", "twenty", "thirty", "fourty", "fifty", "sixty", "seventy", "eighty", "ninety" };

                if (tens == 1)
                {
                    units += 10;
                    tens = 0;
                }

                string LetterUnits = unitsToLetters[units];
                string LetterTens = tensToLetters[tens];
                string LetterHundreds = unitsToLetters[hundreds];


                if (tens == 0 && hundreds == 0 && units != 0)
                    return LetterUnits;

                if (hundreds == 0 && tens != 0 && units != 0)
                    return LetterTens + " " + LetterUnits;

                if (hundreds != 0 && tens == 0 && units != 0)
                    return LetterHundreds + " hundred and " + LetterUnits;

                if (hundreds != 0 && tens == 0 && units == 0)
                    return LetterHundreds + " hundreds";

                if (hundreds != 0 && tens != 0 && units == 0)
                    return LetterHundreds + "hundred and " + LetterTens;

                if (hundreds != 0 && tens != 0 && units != 0)
                    return LetterHundreds + " hundreds " + LetterTens + " " + LetterUnits;

                else return "!error!";
            }

            /// <summary>
            /// Returns an int array. First element is the hours, second is minutes and last is seconds.
            /// </summary>
            /// <param name="value"></param>
            /// <returns></returns>
            public static int[] ToHoursMinutesSeconds(this float value)
            {
                int properTime = Mathf.FloorToInt(value);

                int hours = Mathf.FloorToInt(properTime / 3600);
                int minutes = Mathf.FloorToInt(properTime / 60) - (hours * 60);
                int seconds = properTime - (minutes * 60) - (hours * 3600);

                return new int[] { hours, minutes, seconds };
            }

            /// <summary>
            /// Returns an int array. First element is the hours, second is minutes and last is seconds.
            /// </summary>
            /// <param name="value"></param>
            /// <returns></returns>
            public static int[] ToHoursMinutesSeconds(this double value)
            {
                int properTime = Mathf.FloorToInt((float)value);

                int hours = Mathf.FloorToInt(properTime / 3600);
                int minutes = Mathf.FloorToInt(properTime / 60) - (hours * 60);
                int seconds = properTime - (minutes * 60) - (hours * 3600);

                return new int[] { hours, minutes, seconds };
            }

            /// <summary>
            /// Purges this GameObject from all its components (except Transform). Beware, it may be resource-hungry.
            /// </summary>
            /// <param name="GO"></param>
            public static void Purge(this GameObject GO)
            {
                foreach (var comp in GO.GetComponents<Component>())
                {
                    if (comp.GetType() != typeof(Transform))
                        MonoBehaviour.Destroy(comp);
                }
            }

            /// <summary>
            /// Add this to a GameObject with a mesh to give it a little squishy feeling. Just a little bonus really.
            /// </summary>
            /// <param name="GO"></param>
            public static void ElasticOn(this GameObject GO)
            {
                if (GO.GetComponent<ElasticBehaviour>() == null)
                {
                    GO.AddComponent<ElasticBehaviour>();
                }
                else
                {
                    Debug.LogError(GO.name + " already has an elastic behaviour!");
                }
            }

            /// <summary>
            /// Removes the elastic component from this GameObject, if it has one.
            /// </summary>
            /// <param name="GO"></param>
            public static void ElasticOff(this GameObject GO)
            {
                if (GO.GetComponent<ElasticBehaviour>() != null)
                {
                    MonoBehaviour.Destroy(GO.GetComponent<ElasticBehaviour>());
                }
                else
                {
                    Debug.LogError(GO.name + " has no elastic behaviour but you are trying to remove it!");
                }
            }
        }
    }
}


