﻿/*Copyright (c) 2018 Grégoire Meyer
 * 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 * */

using System.Collections.Generic;
using UnityEngine;
using System;
#if UNITY_EDITOR
using UnityEditor;
using UnityEditorInternal;
#endif

namespace Sigian
{
    #region Structs

    [Serializable]
    /// <summary>
    /// A value float limited by a Min and a Max.
    /// </summary>
    public struct MinMax
    {
        [SerializeField]
        private float Min, Max, Value;

        private bool BreakIfError;

        #region Constructors
        public MinMax(float min, float max, float value, bool breakIfError)
        {
            Min = min;
            Max = max;
            Value = value;
            BreakIfError = breakIfError;

        }

        public MinMax(float min, float max, float value)
        {
            Min = min;
            Max = max;
            Value = value;
            BreakIfError = false;
        }

        public MinMax(float min, float max)
        {
            Min = min;
            Max = max;
            Value = 0f;
            BreakIfError = false;
        }
        #endregion

        /// <summary>
        /// The value that is limited by the min and max.
        /// </summary>
        public float value
        {
            get { return Value; }
            set { if (CheckValueIntegrity(value)) Value = value; }
        }

        /// <summary>
        /// The minimum number that the value can reach.
        /// </summary>
        public float min
        {
            get { return Min; }
            set { if (CheckValueIntegrity(value)) Min = value; }
        }

        /// <summary>
        /// The maximum number that the value can reach.
        /// </summary>
        public float max
        {
            get { return Max; }
            set { if (CheckValueIntegrity(value)) Max = value; }
        }

        /// <summary>
        /// Min and Max in a Vector2 form.
        /// </summary>
        public Vector2 MinMaxVector
        {
            get
            {
                return new Vector2(Min, Max);
            }
            set
            {
                Min = value.x;
                Max = value.y;
            }
        }

        /// <summary>
        /// If true, any problem regarding the MinMax will throw an error and break the script. If false, the value will be ajusted without breaking.
        /// </summary>
        public bool breakIfError
        {
            get { return BreakIfError; }
            set { BreakIfError = value; }
        }

        /// <summary>
        /// Sets the Value to the Minimum.
        /// </summary>
        public void SetToMin()
        {
            Value = Min;
        }

        /// <summary>
        /// Sets the Value to the Maximum.
        /// </summary>
        public void SetToMax()
        {
            Value = Max;
        }

        /// <summary>
        /// Sets the Value to a certain ratio (example 0.5 means halfway between Min and Max).
        /// </summary>
        /// <param name="ratio"></param>
        public void SetToRatio(float ratio)
        {
            if (ratio > 0 && ratio < 1)
            {
                Value = Min + ((Max - Min) * ratio);
            }
        }

        private bool CheckValueIntegrity(float Value)
        {
            if (Max < Min)
            {
                float temp1 = Min;
                float temp2 = Max;
                Min = temp2;
                Max = temp1;

                Debug.LogWarning("Max is inferior to Min! Inverting...");
            }
            if (Value < Min)
            {               
                Debug.LogWarning("Value is inferior to minimum!");
                return false;             
            }

            if (Value > Max)
            {
                Debug.LogWarning("Value is superior to maximum!");
                return false;              
            }

            return true;
        }

    }

    [Serializable]
    public struct SigianReorderableGameObjectList
    {
        [SerializeField]
        private List<GameObject> list;

        #region Constructors
        public SigianReorderableGameObjectList(List<GameObject> listToAdd)
        {
            list = new List<GameObject>();

            foreach (GameObject GO in listToAdd)
            {
                list.Add(GO);
            }
        }
        #endregion

        /// <summary>
        /// Returns the element of the list at the designated index.
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public GameObject GetAtIndex(int index)
        {
            return list[index];
        }

        /// <summary>
        /// Returns the position on the list of a GameObject.
        /// </summary>
        /// <param name="GO">The GameObject you want to get the position of.</param>
        /// <returns>The index of the GameObject.</returns>
        public int GetIndexOf(GameObject GO)
        {
            return list.IndexOf(GO);
        }

        /// <summary>
        /// Clears the list.
        /// </summary>
        public void Clear()
        {
            list.Clear();
        }

        /// <summary>
        /// The length of the list.
        /// </summary>
        public int Count
        {
            get { return list.Count; }
        }
    }

    #endregion

#if UNITY_EDITOR

    #region Drawers

    [CustomPropertyDrawer(typeof(MinMax))]
    public class MinMaxDrwr : PropertyDrawer
    {
        public override void OnGUI(Rect position, SerializedProperty property, GUIContent label)
        {
            position.height *= 0.5f;
            label = EditorGUI.BeginProperty(position, label, property);

            Rect contentPos = EditorGUI.PrefixLabel(position, label);
            contentPos.width *= 0.33f;

            EditorGUI.indentLevel = 0;
            EditorGUIUtility.labelWidth = 35f;
            EditorGUI.PropertyField(contentPos, property.FindPropertyRelative("Min"), new GUIContent("Min"));

            contentPos.x += contentPos.width;
            EditorGUI.LabelField(contentPos, "|Value:" + property.FindPropertyRelative("Value").floatValue.ToString());

            contentPos.x += contentPos.width + 5f;
            EditorGUI.PropertyField(contentPos, property.FindPropertyRelative("Max"), new GUIContent("|Max"));

            contentPos.y += contentPos.height;
            contentPos.x -= contentPos.width * 2;
            contentPos.width *= 3f;

            property.FindPropertyRelative("Value").floatValue = EditorGUI.Slider(contentPos, GUIContent.none, property.FindPropertyRelative("Value").floatValue, property.FindPropertyRelative("Min").floatValue, property.FindPropertyRelative("Max").floatValue);

            EditorGUI.EndProperty();

        }

        public override float GetPropertyHeight(SerializedProperty property, GUIContent label)
        {
            return base.GetPropertyHeight(property, label) * 2;
        }
    }

    [CustomPropertyDrawer(typeof(SigianReorderableGameObjectList), true)]
    public class ReorderableGameObjectListDrawer : PropertyDrawer
    {
        float height;

        private ReorderableList rl;

        private ReorderableList GetRList(SerializedProperty property)
        {
            if (rl == null)
            {
                rl = new ReorderableList(property.serializedObject, property, true, true, true, true);
                rl.drawElementCallback = (Rect rec, int index, bool isActive, bool isFocused) =>
                {
                    rec.width -= 40;
                    rec.x += 20;
                    EditorGUI.PropertyField(rec, property.GetArrayElementAtIndex(index), true);
                };

                rl.drawHeaderCallback = (Rect rect) =>
                {
                    EditorGUI.LabelField(rect, property.name);
                };
            }
            return rl;
        }

        public override void OnGUI(Rect position, SerializedProperty property, GUIContent label)
        {
            var listProperty = property.FindPropertyRelative("list");
            var list = GetRList(listProperty);
            var height = 0f;
            for (var i = 0; i < listProperty.arraySize; i++)
            {
                height = Mathf.Max(height, EditorGUI.GetPropertyHeight(listProperty.GetArrayElementAtIndex(i)));
            }
            list.elementHeight = height;
            list.DoList(position);
        }


        public override float GetPropertyHeight(SerializedProperty property, GUIContent label)
        {
            return GetRList(property.FindPropertyRelative("list")).GetHeight();
        }

    }

    #endregion

#endif
}
